@@ -1,9 +1,16 @@
-"""Modification of the Sphinx HTML5 translator with better handling of permalinks.
+"""Modification of the Sphinx HTML5 translator.
 
-Instead of "permalink to this headline",
-this returns "Copy link to section: *sectionname*".
-Clicking on the permalink anchor will copy the link to the clipboard.
-This is implemented in JavaScript.
+Overwrite several methods to improve the HTML output.
+
+Improve headerlinks
+   Instead of writing "permalink to this headline", we write
+   "Copy link to section: <SECTIONNAME>" and display an icon
+   instead of the '¶' character.
+
+Semantic HTML5 tags
+   Instead of <div class="section"> we'll use <section>
+
+Remove unnecessary nesting
 
 :copyright: Copyright Kai Welke.
 :license: MIT, see LICENSE for details.
@@ -11,64 +18,34 @@ This is implemented in JavaScript.
 from typing import Any, Dict
 
 from docutils import nodes
-from docutils.nodes import Element, Text
+from docutils.nodes import Element
+from sphinx import addnodes
 from sphinx.application import Sphinx
 from sphinx.locale import _
 from sphinx.util import logging
 from sphinx.writers.html5 import HTML5Translator
 
 from . import __version__
+from .icons import ICONS
 
 logger = logging.getLogger(__name__)
 
-# icons are from Material Design icon set
-ICONS = {
-    # https://material.io/resources/icons/?icon=content_copy
-    "copy": (
-        "<svg xmlns='http://www.w3.org/2000/xvg' viewBox='0 0 24 24' "
-        "fill=currentColor aria-hidden='true'>"
-        "<path d='M16 1H4c-1.1 0-2 .9-2 2v14h2V3h12V1zm3 "
-        "4H8c-1.1 0-2 .9-2 2v14c0 1.1.9 2 2 2h11c1.1 "
-        "0 2-.9 2-2V7c0-1.1-.9-2-2-2zm0 16H8V7h11v14z' /></svg>"
-    ),
-    # https://material.io/resources/icons/?icon=link
-    "headerlink": (
-        '<svg xmlns="http://www.w3.org/2000/svg" '
-        'pointer-events="none" viewBox="0 0 24 24">'
-        '<path d="M3.9 12c0-1.71 1.39-3.1 '
-        "3.1-3.1h4V7H7c-2.76 0-5 2.24-5 5s2.24 "
-        "5 5 5h4v-1.9H7c-1.71 0-3.1-1.39-3.1-3.1zM8 "
-        "13h8v-2H8v2zm9-6h-4v1.9h4c1.71 0 3.1 1.39 3.1 "
-        "3.1s-1.39 3.1-3.1 3.1h-4V17h4c2.76 0 5-2.24 "
-        '5-5s-2.24-5-5-5z"/></svg>'
-    ),
-    # https://material.io/resources/icons/?icon=open_in_new
-    "external_link": (
-        '<svg xmlns="http://www.w3.org/2000/svg" '
-        'viewBox="0 0 24 24" class="external-link-icon" aria-hidden="true">'
-        '<path d="M19 19H5V5h7V3H5a2 2 0 00-2 2v14a2 2 0 '
-        "002 2h14c1.1 0 2-.9 2-2v-7h-2v7zM14 "
-        '3v2h3.59l-9.83 9.83 1.41 1.41L19 6.41V10h2V3h-7z"/></svg>'
-    ),
-}
-
 COPY_BUTTON = (
     "<button class='copy tooltipped tooltipped-nw' aria-label='Copy this code'>"
     + ICONS["copy"]
     + "</button>\n"
 )
 
+EXPAND_MORE_BUTTON = (
+    "<button class='expand-more tooltipped tooltipped-nw' "
+    "aria-label='Expand this section' aria-expanded='false'>"
+    + ICONS["expand_more"]
+    + "</button>"
+)
+
 
 class AwesomeHTMLTranslator(HTML5Translator):
-    """Override a few methods to improve the usability.
-
-    By default, Sphinx permalinks have generic titles,
-    such as: "Permalink to this section".
-    In the sphinx awesome theme, clicking on a permalink
-    copies the link to the clipboard. Therefore,
-    the link title attributes are updated to be more
-    specific.
-    """
+    """Override a few methods to improve the usability."""
 
     def depart_title(self, node: Element) -> None:
         """Change the permalinks for headlines.
@@ -88,11 +65,22 @@ class AwesomeHTMLTranslator(HTML5Translator):
             and node.parent.hasattr("ids")
             and node.parent["ids"]
         ):
-            # add permalink anchor
+            # add permalink anchor to normal headings
             if close_tag.startswith("</h"):
                 self.add_permalink_ref(
                     node.parent, _(f"Copy link to section: {node.astext()}.")
                 )
+            # and to headings when the 'contents' directive is used
+            elif close_tag.startswith("</a></h"):
+                self.body.append(
+                    "</a><a role='button' "
+                    "class='headerlink tooltipped tooltipped-ne' "
+                    'href="#{}" '
+                    'aria-label="Copy link to this section: {}">'.format(
+                        node.parent["ids"][0], node.astext()
+                    )
+                    + ICONS["headerlink"]
+                )
             elif isinstance(node.parent, nodes.table):
                 self.body.append("</span>")
                 self.add_permalink_ref(node.parent, _("Copy link to this table."))
@@ -176,6 +164,29 @@ class AwesomeHTMLTranslator(HTML5Translator):
         else:
             self.body.append("</p>\n")
 
+    def visit_desc(self, node: Element) -> None:
+        """Add a class ``code-definition`` to definition lists.
+
+        For code objects, like functions, classes, methods, etc.
+        to distinguish them from regular definition lists.
+        """
+        cl = node["objtype"] + " code-definition"
+        self.body.append(self.starttag(node, "dl", CLASS=cl))
+
+    def visit_desc_signature(self, node: Element) -> None:
+        """Add the accordion class to the <dt> element.
+
+        This will make the definition list collapsible,
+        if the configuration option ``html_collapsible_definitions``
+        is set.
+        """
+        # only add this, if the following dd is not empty.
+        dd = node.next_node(addnodes.desc_content, siblings=True)
+        if self.config.html_collapsible_definitions and len(dd.astext()) > 0:
+            self.body.append(self.starttag(node, "dt", CLASS="accordion"))
+        else:
+            self.body.append(self.starttag(node, "dt"))
+
     def depart_desc_signature(self, node: Element) -> None:
         """Change permalinks for code definitions.
 
@@ -184,14 +195,30 @@ class AwesomeHTMLTranslator(HTML5Translator):
         """
         if not node.get("is_multiline"):
             self.add_permalink_ref(node, _("Copy link to this definition."))
+        dd = node.next_node(addnodes.desc_content, siblings=True)
+        if self.config.html_collapsible_definitions and len(dd.astext()) > 0:
+            self.body.append(EXPAND_MORE_BUTTON)
         self.body.append("</dt>\n")
 
     def depart_desc_signature_line(self, node: Element) -> None:
-        """Change permalinks for code definitions."""
+        """Change permalinks for code definitions.
+
+        This method is only relevant for mulitline definitions,
+        which I think only happen in C and C++ domains.
+        """
         if node.get("add_permalink"):
             self.add_permalink_ref(node.parent, _("Copy link to this definition."))
+        if self.config.html_collapsible_definitions:
+            self.body.append(EXPAND_MORE_BUTTON)
         self.body.append("<br />")
 
+    def visit_desc_content(self, node: Element) -> None:
+        """Add panel class to definitions."""
+        if self.config.html_collapsible_definitions and len(node.astext()) > 0:
+            self.body.append(self.starttag(node, "dd", CLASS="panel"))
+        else:
+            self.body.append(self.starttag(node, "dd"))
+
     def visit_section(self, node: Element) -> None:
         """Use semantic <section> elements."""
         self.section_level += 1
@@ -206,8 +233,9 @@ class AwesomeHTMLTranslator(HTML5Translator):
         """Use semantic <figure> elements."""
         attributes = {}
         if node.get("width"):
-            attributes["style"] = f"width:{node['width']}"
-        if node.get("align"):
+            attributes["style"] = f"width: {node['width']}"
+        # in Sphinx this is always set
+        if node.get("align"):  # pragma: nocover
             attributes["class"] = f"align-{node['align']}"
         self.body.append(self.starttag(node, "figure", **attributes))
 
@@ -216,100 +244,98 @@ class AwesomeHTMLTranslator(HTML5Translator):
         self.body.append("</figure>\n")
 
     def visit_literal_block(self, node: Element) -> None:
-        """Don't wrap highlighted blocks any further."""
-        if node.rawsource != node.astext():
-            # most probably a parsed-literal block -- don't highlight
-            return super().visit_literal_block(node)
+        """Overwrite code blocks.
 
-        lang = node.get("language", "default")
-        linenos = node.get("linenos", False)
-        highlight_args = node.get("highlight_args", {})
-        highlight_args["force"] = node.get("force", False)
-        if lang is self.builder.config.highlight_language:
-            # only pass highlighter options for original language
-            opts = self.builder.config.highlight_options
+        All code blocks have a header with at least a copy button.
+        For code blocks with syntax highlighting, the language is
+        shown on the left side and an optional caption is included
+        in the center.
+        """
+        if node.rawsource == node.astext():
+            # node doens't have markup, highlight it!
+            lang = node.get("language", "default")
+            linenos = node.get("linenos", False)
+            highlight_args = node.get("highlight_args", {})
+            highlight_args["force"] = node.get("force", False)
+
+            if lang is self.builder.config.highlight_language:
+                # only pass highlighter options for original language
+                opts = self.builder.config.highlight_options
+            else:
+                opts = {}
+
+            if linenos and self.builder.config.html_codeblock_linenos_style:
+                linenos = "inline"
+
+            highlighted = self.highlighter.highlight_block(
+                node.rawsource,
+                lang,
+                opts=opts,
+                linenos=linenos,
+                location=node,
+                **highlight_args,
+            )
+
+            # Code blocks that don't have a caption are not wrapped inside a <container>
+            # node so we add the header here. With captions, see: visit_caption
+            if not (
+                isinstance(node.parent, nodes.container)
+                and node.parent.get("literal_block")
+            ):
+
+                self.body.append(self.starttag(node, "div", CLASS="highlight"))
+
+                code_header = "<div class='code-header'>\n"
+                code_lang = lang.replace("default", "python").replace(
+                    "console", "shell"
+                )
+                code_header += f"<span class='code-lang'>{code_lang}</span>\n"
+                code_header += COPY_BUTTON
+                code_header += "</div>\n"
+                self.body.append(code_header)
+
+            # wrap the highlighted string in a div
+            self.body.append(highlighted)
+
+            if not (
+                isinstance(node.parent, nodes.container)
+                and node.parent.get("literal_block")
+            ):
+                self.body.append("</div>\n")
+            # we already included everything here in the `highlighted` string,
+            # so we need to skip further processing
+            raise nodes.SkipNode
         else:
-            opts = {}
-
-        if linenos and self.builder.config.html_codeblock_linenos_style:
-            linenos = "inline"
-
-        highlighted = self.highlighter.highlight_block(
-            node.rawsource,
-            lang,
-            opts=opts,
-            linenos=linenos,
-            location=node,
-            **highlight_args,
-        )
-
-        # if the node is not child of a container.literal_block node
-        if not (
-            isinstance(node.parent, nodes.container)
-            and node.parent.get("literal_block")
-        ):
-
+            # node has markup, it's a samp directive or parsed-literal
             self.body.append(self.starttag(node, "div", CLASS="highlight"))
-
             code_header = "<div class='code-header'>\n"
-            code_lang = lang.replace("default", "python").replace("console", "shell")
-            code_header += f"<span class='code-lang'>{code_lang}</span>\n"
             code_header += COPY_BUTTON
             code_header += "</div>\n"
             self.body.append(code_header)
+            self.body.append("<pre><code>")
 
-        # wrap the highlighted string in a div
-        self.body.append(highlighted)
+    def depart_literal_block(self, node: Element) -> None:
+        """Close literal blocks.
 
-        if not (
-            isinstance(node.parent, nodes.container)
-            and node.parent.get("literal_block")
-        ):
-            self.body.append("</div>\n")
-        raise nodes.SkipNode
-
-    def visit_literal(self, node: Element) -> None:
-        """Simplify inline code elements."""
-        if "kbd" in node["classes"]:
-            self.body.append(self.starttag(node, "kbd"))
-        else:
-            self.body.append(self.starttag(node, "code"))
-
-        self.protect_literal_text += 1
-
-    def depart_literal(self, node: Element) -> None:
-        """Simplify inline code elements."""
-        if "kbd" in node["classes"]:
-            self.body.append("</kbd>")
-        else:
-            self.body.append("</code>")
-
-    def visit_Text(self, node: Text) -> None:
-        """Simplify inline code elements."""
-        text = node.astext()
-        encoded = self.encode(text)
-
-        if self.protect_literal_text:
-            for token in self.words_and_spaces.findall(encoded):
-                if token.strip() or token in " \n":  # noqa: S105
-                    self.body.append(token)
-                else:
-                    self.body.append("&#160;" * (len(token) - 1) + " ")
-        else:
-            if self.in_mailto and self.settings.cloak_email_addresses:
-                encoded = self.cloak_email(encoded)
-            self.body.append(encoded)
+        We need to provide the closing tag for non-highlighted
+        code blocks. This method is skipped (``raise nodes.SkipNode``)
+        for highlighted code blocks.
+        """
+        self.body.append("</code></pre>\n")
+        self.body.append("</div>\n")
 
     def visit_container(self, node: Element) -> None:
         """Overide for code blocks with captions."""
-        self.body.append(self.starttag(node, "div", CLASS="highlight"))
         if node.get("literal_block"):
+            self.body.append(self.starttag(node, "div", CLASS="highlight"))
             lang = node.get("language")
             code_header = "<div class='code-header'>\n"
             code_header += (
                 f"<span class='code-lang'>{lang.replace('default', 'python')}</span>\n"
             )
             self.body.append(code_header)
+        else:
+            super().visit_container(node)
 
     def depart_reference(self, node: Element) -> None:
         """Add external link icon."""
@@ -325,6 +351,7 @@ def setup(app: "Sphinx") -> Dict[str, Any]:
     """
     app.set_translator("html", AwesomeHTMLTranslator)
     app.set_translator("dirhtml", AwesomeHTMLTranslator)
+    app.add_config_value("html_collapsible_definitions", False, "html")
 
     return {
         "version": __version__,
